import 'package:flutter/material.dart';
import 'package:bootstrap_like_breakpoints/bootstrap_like_breakpoints.dart';
import 'package:bootstrap_like_grid/src/wrappers/bs_container_wrapper.dart';

/// A Bootstrap like container. Can be fluid = always screen width.
/// Default is to be sized based on the screen size, see breakpoints for
/// definitions. Also, can have a max breakpoint specified.
/// ""
/// sm
/// md
/// lg
/// xl
/// xxl
/// The max breakpoint is the largest the container can be
/// note that a breakpoint can't be used if fluid = true
/// *** Should be the child of a Material App ***
class BSContainer extends StatefulWidget {
  /// set container to screen size
  final bool fluid;

  /// max breakpoint, will not be bigger than. Can't be fluid to
  final String? maxBreakPoint;

  /// children of the container
  final List<Widget> children;

  final double? height;
  final Alignment alignment;
  final Clip clipBehavior;
  final BoxConstraints? boxConstraints;
  final Decoration? decoration;
  final Color? color;
  final Decoration? foregroundDecoration;
  final EdgeInsetsGeometry? margin;
  final EdgeInsetsGeometry? padding;
  final Matrix4? transform;
  final AlignmentGeometry? transformAlignment;

  BSContainer({
    Key? key,
    this.fluid = false,
    this.maxBreakPoint,
    required this.children,
    this.height,
    this.alignment = Alignment.topCenter,
    this.clipBehavior = Clip.none,
    this.boxConstraints,
    this.decoration,
    this.color,
    this.foregroundDecoration,
    this.margin,
    this.padding,
    this.transform,
    this.transformAlignment,
  }) : super(key: key) {
    // confirm that the passed breakpoint is valid
    if (!BSBreakPoints.getBreakPointLabels().contains(maxBreakPoint) &&
        maxBreakPoint != null) {
      throw FormatException(
          "BSContainer:\nInvalid break point passed. Must be (lowercase):\n${BSBreakPoints.getBreakPointLabels().toString()}");
    }
  }

  @override
  _BSContainerState createState() => _BSContainerState();
}

class _BSContainerState extends State<BSContainer> {
  /// if fluid width = screen size
  /// otherwise use breakpoint definitions and screen size to
  /// determine the width
  double _calculateContainerWidth(BuildContext context) {
    // fluid
    if (widget.fluid) {
      return MediaQuery.of(context).size.width;

      // limit to max breakpoint
    } else if (widget.maxBreakPoint != null) {
      // generate list of possible widths
      List<double> widths = <double>[
        MediaQuery.of(context).size.width,
        BSBreakPoints.getContainerWidth("sm"),
        BSBreakPoints.getContainerWidth("md"),
        BSBreakPoints.getContainerWidth("lg"),
        BSBreakPoints.getContainerWidth("xl"),
        BSBreakPoints.getContainerWidth("xxl"),
      ];

      // all widths above max breakpoint set to max breakpoint width
      List<String> breakPointLabels = BSBreakPoints.getBreakPointLabels();
      for (int i = breakPointLabels.indexOf(widget.maxBreakPoint!) + 1;
          i < breakPointLabels.length;
          i++) {
        widths[i] = widths[i - 1];
      }

      // return width based on current breakpoint
      return BSBreakPoints.valueBasedOnBreakPoint(
        context: context,
        map: <String, double>{
          breakPointLabels[0]: widths[0],
          breakPointLabels[1]: widths[1],
          breakPointLabels[2]: widths[2],
          breakPointLabels[3]: widths[3],
          breakPointLabels[4]: widths[4],
          breakPointLabels[5]: widths[5],
        },
      );

      // default breakpoints
    } else {
      return BSBreakPoints.valueBasedOnBreakPoint(
        context: context,
        map: <String, double>{
          "": MediaQuery.of(context).size.width,
          "sm": BSBreakPoints.getContainerWidth("sm"),
          "md": BSBreakPoints.getContainerWidth("md"),
          "lg": BSBreakPoints.getContainerWidth("lg"),
          "xl": BSBreakPoints.getContainerWidth("xl"),
          "xxl": BSBreakPoints.getContainerWidth("xxl"),
        },
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    double width = _calculateContainerWidth(context);
    return SingleChildScrollView(
      child: BSContainerWrapper(
        child: Container(
          alignment: widget.alignment,
          clipBehavior: widget.clipBehavior,
          constraints: widget.boxConstraints,
          decoration: widget.decoration,
          color: widget.color,
          foregroundDecoration: widget.foregroundDecoration,
          margin: widget.margin,
          padding: widget.padding,
          transform: widget.transform,
          transformAlignment: widget.transformAlignment,
          child: SizedBox(
            width: width,
            height: widget.height,
            child: Column(
              children: widget.children,
            ),
          ),
        ),
        width: width,
      ),
    );
  }
}
